@@ -1,4 +1,3 @@
-import base64
 import json
 import logging
 import re
@@ -25,6 +24,7 @@ from localstack.services.apigateway.helpers import (
     make_error_response,
 )
 from localstack.services.apigateway.integration import (
+    LambdaProxyIntegration,
     MockIntegration,
     RequestTemplates,
     ResponseTemplates,
@@ -42,7 +42,7 @@ from localstack.utils.aws.aws_responses import (
     request_response_stream,
     requests_response,
 )
-from localstack.utils.common import camel_to_snake_case, json_safe, to_bytes, to_str
+from localstack.utils.common import camel_to_snake_case, json_safe
 
 # set up logger
 from localstack.utils.http import add_query_params_to_url
@@ -269,16 +269,16 @@ def invoke_rest_api(invocation_context: ApiInvocationContext):
     if not is_api_key_valid(api_key_required, headers, invocation_context.stage):
         return make_error_response("Access denied - invalid API key", 403)
 
-    integrations = resource.get("resourceMethods", {})
-    integration = integrations.get(method, {})
-    if not integration:
+    resource_methods = resource.get("resourceMethods", {})
+    resource_method = resource_methods.get(method, {})
+    if not resource_method:
         # HttpMethod: '*'
         # ResourcePath: '/*' - produces 'X-AMAZON-APIGATEWAY-ANY-METHOD'
-        integration = integrations.get("ANY", {}) or integrations.get(
+        resource_method = resource_methods.get("ANY", {}) or resource_methods.get(
             "X-AMAZON-APIGATEWAY-ANY-METHOD", {}
         )
-    integration = integration.get("methodIntegration")
-    if not integration:
+    method_integration = resource_method.get("methodIntegration")
+    if not method_integration:
         if method == "OPTIONS" and "Origin" in headers:
             # default to returning CORS headers if this is an OPTIONS request
             return get_cors_response(headers)
@@ -287,16 +287,9 @@ def invoke_rest_api(invocation_context: ApiInvocationContext):
             404,
         )
 
-    res_methods = resource.get("resourceMethods", {})
-    meth_integration = res_methods.get(method, {}).get("methodIntegration", {})
-    int_responses = meth_integration.get("integrationResponses", {})
-    response_templates = int_responses.get("200", {}).get("responseTemplates", {})
-
     # update fields in invocation context, then forward request to next handler
-    invocation_context.resource = resource
     invocation_context.resource_path = extracted_path
-    invocation_context.response_templates = response_templates
-    invocation_context.integration = integration
+    invocation_context.integration = method_integration
 
     return invoke_rest_api_integration(invocation_context)
 
@@ -326,9 +319,9 @@ def invoke_rest_api_integration_backend(invocation_context: ApiInvocationContext
     api_id = invocation_context.api_id
     stage = invocation_context.stage
     resource_path = invocation_context.resource_path
-    response_templates = invocation_context.response_templates
     integration = invocation_context.integration
-
+    integration_response = integration.get("integrationResponses", {})
+    response_templates = integration_response.get("200", {}).get("responseTemplates", {})
     # extract integration type and path parameters
     relative_path, query_string_params = extract_query_string_params(path=invocation_path)
     integration_type_orig = integration.get("type") or integration.get("integrationType") or ""
@@ -348,13 +341,16 @@ def invoke_rest_api_integration_backend(invocation_context: ApiInvocationContext
     if (uri.startswith("arn:aws:apigateway:") and ":lambda:path" in uri) or uri.startswith(
         "arn:aws:lambda"
     ):
-        if integration_type in ["AWS", "AWS_PROXY"]:
+        if integration_type == "AWS_PROXY":
+            return LambdaProxyIntegration().invoke(invocation_context)
+        elif integration_type == "AWS":
             func_arn = uri
             if ":lambda:path" in uri:
                 func_arn = (
                     uri.split(":lambda:path")[1].split("functions/")[1].split("/invocations")[0]
                 )
 
+            headers = helpers.create_invocation_headers(invocation_context)
             invocation_context.context = helpers.get_event_request_context(invocation_context)
             invocation_context.stage_variables = helpers.get_stage_variables(invocation_context)
             if invocation_context.authorizer_type:
@@ -391,24 +387,9 @@ def invoke_rest_api_integration_backend(invocation_context: ApiInvocationContext
                 )
                 parsed_result = common.json_safe(parsed_result)
                 parsed_result = {} if parsed_result is None else parsed_result
-                response.status_code = int(parsed_result.get("statusCode", 200))
-                parsed_headers = parsed_result.get("headers", {})
-                if parsed_headers is not None:
-                    response.headers.update(parsed_headers)
-                try:
-                    result_body = parsed_result.get("body")
-                    if isinstance(result_body, dict):
-                        response._content = json.dumps(result_body)
-                    else:
-                        body_bytes = to_bytes(to_str(result_body or ""))
-                        if parsed_result.get("isBase64Encoded", False):
-                            body_bytes = base64.b64decode(body_bytes)
-                        response._content = body_bytes
-                except Exception as e:
-                    LOG.warning("Couldn't set Lambda response content: %s", e)
-                    response._content = "{}"
+                response.status_code = 200
+                response._content = parsed_result
                 update_content_length(response)
-                response.multi_value_headers = parsed_result.get("multiValueHeaders") or {}
 
             # apply custom response template
             invocation_context.response = response
@@ -644,7 +625,10 @@ def invoke_rest_api_integration_backend(invocation_context: ApiInvocationContext
             payload = json.dumps(payload)
 
         uri = apply_request_parameters(
-            uri, integration=integration, path_params=path_params, query_params=query_string_params
+            uri,
+            integration=integration,
+            path_params=path_params,
+            query_params=query_string_params,
         )
         result = requests.request(method=method, url=uri, data=payload, headers=headers)
         # apply custom response template
