@@ -13,8 +13,7 @@ import traceback
 import uuid
 from datetime import datetime
 from io import StringIO
-from threading import BoundedSemaphore
-from typing import Any, Dict, List, Optional, Tuple, Type
+from typing import Any, Dict, List, Optional, Tuple
 from urllib.parse import urlparse
 
 from flask import Flask, Response, jsonify, request
@@ -22,13 +21,15 @@ from flask import Flask, Response, jsonify, request
 from localstack import config
 from localstack.constants import APPLICATION_JSON, LAMBDA_TEST_ROLE, TEST_AWS_ACCOUNT_ID
 from localstack.services.awslambda import lambda_executors
+from localstack.services.awslambda.event_source_listeners.event_source_listener import (
+    EventSourceListener,
+)
 from localstack.services.awslambda.lambda_executors import InvocationResult, LambdaContext
 from localstack.services.awslambda.lambda_utils import (
     API_PATH_ROOT,
     DOTNET_LAMBDA_RUNTIMES,
     LAMBDA_DEFAULT_HANDLER,
     LAMBDA_DEFAULT_RUNTIME,
-    LAMBDA_DEFAULT_STARTING_POSITION,
     LAMBDA_RUNTIME_NODEJS14X,
     ClientError,
     error_response,
@@ -41,7 +42,6 @@ from localstack.services.awslambda.lambda_utils import (
 )
 from localstack.services.generic_proxy import RegionBackend
 from localstack.services.install import INSTALL_DIR_STEPFUNCTIONS, install_go_lambda_runtime
-from localstack.utils import bootstrap
 from localstack.utils.analytics import event_publisher
 from localstack.utils.aws import aws_stack
 from localstack.utils.aws.aws_models import CodeSigningConfig, LambdaFunction
@@ -65,19 +65,17 @@ from localstack.utils.common import (
     safe_requests,
     save_file,
     short_uid,
-    start_worker_thread,
     synchronized,
     timestamp_millis,
     to_bytes,
     to_str,
     unzip,
 )
+from localstack.utils.container_networking import get_main_container_name
 from localstack.utils.docker_utils import DOCKER_CLIENT
 from localstack.utils.functions import run_safe
-from localstack.utils.generic.singleton_utils import SubtypesInstanceManager
 from localstack.utils.http import canonicalize_headers, parse_chunked_data
 from localstack.utils.patch import patch
-from localstack.utils.run import FuncThread
 
 # logger
 LOG = logging.getLogger(__name__)
@@ -103,7 +101,10 @@ BATCH_SIZE_RANGES = {
     "kafka": (100, 10000),
     "kinesis": (100, 10000),
     "dynamodb": (100, 1000),
-    "sqs": (10, 10),
+    "sqs": (
+        10,
+        10,
+    ),  # should be (10,10000) for normal SQS queues, (10,10) for FIFO https://docs.aws.amazon.com/lambda/latest/dg/API_CreateEventSourceMapping.html#SSS-CreateEventSourceMapping-request-BatchSize
 }
 
 DATE_FORMAT = "%Y-%m-%dT%H:%M:%S.%f+00:00"
@@ -165,229 +166,6 @@ class LambdaRegion(RegionBackend):
         self.event_source_mappings = []
 
 
-class EventSourceListener(SubtypesInstanceManager):
-    INSTANCES: Dict[str, "EventSourceListener"] = {}
-
-    @staticmethod
-    def source_type() -> str:
-        """Type discriminator - to be implemented by subclasses."""
-        raise NotImplementedError
-
-    def start(self):
-        """Start listener in the background (for polling mode) - to be implemented by subclasses."""
-        pass
-
-    def process_event(self, event: Any):
-        """Process the given event (for reactive mode)"""
-        pass
-
-    @staticmethod
-    def start_listeners(event_source_mapping: Dict):
-        source_arn = event_source_mapping.get("EventSourceArn") or ""
-        parts = source_arn.split(":")
-        service_type = parts[2] if len(parts) > 2 else ""
-        if not service_type:
-            self_managed_endpoints = event_source_mapping.get("SelfManagedEventSource", {}).get(
-                "Endpoints", {}
-            )
-            if self_managed_endpoints.get("KAFKA_BOOTSTRAP_SERVERS"):
-                service_type = "kafka"
-        instance = EventSourceListener.get(service_type, raise_if_missing=False)
-        if instance:
-            instance.start()
-
-    @staticmethod
-    def process_event_via_listener(service_type: str, event: Any):
-        """Process event for the given service type (for reactive mode)"""
-        instance = EventSourceListener.get(service_type, raise_if_missing=False)
-        if not instance:
-            return
-
-        def _process(*args):
-            instance.process_event(event)
-
-        # start processing in background
-        start_worker_thread(_process)
-
-    @classmethod
-    def impl_name(cls) -> str:
-        return cls.source_type()
-
-    @classmethod
-    def get_base_type(cls) -> Type:
-        return EventSourceListener
-
-
-class EventSourceListenerSQS(EventSourceListener):
-    # SQS listener thread settings
-    SQS_LISTENER_THREAD: Dict = {}
-    SQS_POLL_INTERVAL_SEC: float = 1
-    # Whether to use polling via SQS API (or, alternatively, reactive mode with SQS updates received directly in-memory)
-    # Advantage of polling is that we can delete messages directly from the queue (via 'ReceiptHandle') after processing
-    USE_POLLING = True
-
-    @staticmethod
-    def source_type():
-        return "sqs"
-
-    def start(self):
-        if not self.USE_POLLING:
-            return
-        if self.SQS_LISTENER_THREAD:
-            return
-
-        LOG.debug("Starting SQS message polling thread for Lambda API")
-        self.SQS_LISTENER_THREAD["_thread_"] = thread = FuncThread(self._listener_loop)
-        thread.start()
-
-    def get_matching_event_sources(self) -> List[Dict]:
-        return get_event_sources(source_arn=r".*:sqs:.*")
-
-    def process_event(self, event: Any):
-        if self.USE_POLLING:
-            return
-        # feed message into the first listening lambda (message should only get processed once)
-        queue_url = event["QueueUrl"]
-        try:
-            queue_name = queue_url.rpartition("/")[2]
-            queue_arn = aws_stack.sqs_queue_arn(queue_name)
-            sources = get_event_sources(source_arn=queue_arn)
-            arns = [s.get("FunctionArn") for s in sources]
-            source = (sources or [None])[0]
-            if not source:
-                return False
-
-            LOG.debug(
-                "Found %s source mappings for event from SQS queue %s: %s",
-                len(arns),
-                queue_arn,
-                arns,
-            )
-            # TODO: support message BatchSize here, same as for polling mode below
-            messages = event["Messages"]
-            self._process_messages_for_event_source(source, messages)
-        except Exception:
-            LOG.exception(f"Unable to run Lambda function on SQS messages from queue {queue_url}")
-
-    def _listener_loop(self, *args):
-        while True:
-            try:
-                sources = self.get_matching_event_sources()
-                if not sources:
-                    # Temporarily disable polling if no event sources are configured
-                    # anymore. The loop will get restarted next time a message
-                    # arrives and if an event source is configured.
-                    self.SQS_LISTENER_THREAD.pop("_thread_")
-                    return
-
-                unprocessed_messages = {}
-
-                for source in sources:
-                    queue_arn = source["EventSourceArn"]
-                    region_name = queue_arn.split(":")[3]
-                    sqs_client = aws_stack.connect_to_service("sqs", region_name=region_name)
-                    batch_size = max(min(source.get("BatchSize", 1), 10), 1)
-
-                    try:
-                        queue_url = aws_stack.sqs_queue_url_for_arn(queue_arn)
-                        messages = unprocessed_messages.pop(queue_arn, None)
-                        if not messages:
-                            result = sqs_client.receive_message(
-                                QueueUrl=queue_url,
-                                AttributeNames=["All"],
-                                MessageAttributeNames=["All"],
-                                MaxNumberOfMessages=batch_size,
-                            )
-                            messages = result.get("Messages")
-                            if not messages:
-                                continue
-
-                        res = self._process_messages_for_event_source(source, messages)
-                        if not res:
-                            unprocessed_messages[queue_arn] = messages
-
-                    except Exception as e:
-                        if "NonExistentQueue" not in str(e):
-                            # TODO: remove event source if queue does no longer exist?
-                            LOG.debug("Unable to poll SQS messages for queue %s: %s", queue_arn, e)
-
-            except Exception:
-                pass
-            finally:
-                time.sleep(self.SQS_POLL_INTERVAL_SEC)
-
-    def _process_messages_for_event_source(self, source, messages):
-        lambda_arn = source["FunctionArn"]
-        queue_arn = source["EventSourceArn"]
-        region_name = queue_arn.split(":")[3]
-        queue_url = aws_stack.sqs_queue_url_for_arn(queue_arn)
-        LOG.debug("Sending event from event source %s to Lambda %s", queue_arn, lambda_arn)
-        res = self._send_event_to_lambda(
-            queue_arn,
-            queue_url,
-            lambda_arn,
-            messages,
-            region=region_name,
-        )
-        return res
-
-    def _send_event_to_lambda(self, queue_arn, queue_url, lambda_arn, messages, region):
-        def delete_messages(result, func_arn, event, error=None, dlq_sent=None, **kwargs):
-            if error and not dlq_sent:
-                # Skip deleting messages from the queue in case of processing errors AND if
-                # the message has not yet been sent to a dead letter queue (DLQ).
-                # We'll pick them up and retry next time they become available on the queue.
-                return
-
-            region_name = queue_arn.split(":")[3]
-            sqs_client = aws_stack.connect_to_service("sqs", region_name=region_name)
-            entries = [
-                {"Id": r["receiptHandle"], "ReceiptHandle": r["receiptHandle"]} for r in records
-            ]
-            try:
-                sqs_client.delete_message_batch(QueueUrl=queue_url, Entries=entries)
-            except Exception as e:
-                LOG.info(
-                    "Unable to delete Lambda events from SQS queue "
-                    + "(please check SQS visibility timeout settings): %s - %s" % (entries, e)
-                )
-
-        records = []
-        for msg in messages:
-            message_attrs = message_attributes_to_lower(msg.get("MessageAttributes"))
-            records.append(
-                {
-                    "body": msg.get("Body", "MessageBody"),
-                    "receiptHandle": msg.get("ReceiptHandle"),
-                    "md5OfBody": msg.get("MD5OfBody") or msg.get("MD5OfMessageBody"),
-                    "eventSourceARN": queue_arn,
-                    "eventSource": lambda_executors.EVENT_SOURCE_SQS,
-                    "awsRegion": region,
-                    "messageId": msg["MessageId"],
-                    "attributes": msg.get("Attributes", {}),
-                    "messageAttributes": message_attrs,
-                    "md5OfMessageAttributes": msg.get("MD5OfMessageAttributes"),
-                    "sqs": True,
-                }
-            )
-
-        event = {"Records": records}
-
-        # TODO implement retries, based on "RedrivePolicy.maxReceiveCount" in the queue settings
-        res = run_lambda(
-            func_arn=lambda_arn,
-            event=event,
-            context={},
-            asynchronous=True,
-            callback=delete_messages,
-        )
-        if isinstance(res, InvocationResult):
-            status_code = getattr(res.result, "status_code", 0)
-            if status_code >= 400:
-                return False
-        return True
-
-
 def cleanup():
     region = LambdaRegion.get()
     region.lambdas = {}
@@ -443,6 +221,7 @@ def build_mapping_obj(data) -> Dict:
     mapping["State"] = "Enabled" if enabled in [True, None] else "Disabled"
     mapping["ParallelizationFactor"] = data.get("ParallelizationFactor") or 1
     mapping["Topics"] = data.get("Topics") or []
+    mapping["MaximumRetryAttempts"] = data.get("MaximumRetryAttempts") or -1
     if "SelfManagedEventSource" in data:
         source_arn = data["SourceAccessConfigurations"][0]["URI"]
         mapping["SelfManagedEventSource"] = data["SelfManagedEventSource"]
@@ -450,9 +229,13 @@ def build_mapping_obj(data) -> Dict:
     else:
         source_arn = data["EventSourceArn"]
         mapping["EventSourceArn"] = source_arn
-        mapping["StartingPosition"] = LAMBDA_DEFAULT_STARTING_POSITION
+        mapping["StartingPosition"] = data.get("StartingPosition") or "LATEST"
     batch_size = check_batch_size_range(source_arn, batch_size)
     mapping["BatchSize"] = batch_size
+
+    if data.get("DestinationConfig"):
+        mapping["DestinationConfig"] = data.get("DestinationConfig")
+
     return mapping
 
 
@@ -566,8 +349,6 @@ def process_apigateway_invocation(
 ):
     if path_params is None:
         path_params = {}
-    if stage_variables is None:
-        stage_variables = {}
     if request_context is None:
         request_context = {}
     try:
@@ -580,15 +361,14 @@ def process_apigateway_invocation(
         event["pathParameters"] = path_params
         event["resource"] = resource_path
         event["requestContext"] = request_context
-        if stage_variables:
-            event["stageVariables"] = stage_variables
+        event["stageVariables"] = stage_variables
         LOG.debug(
             "Running Lambda function %s from API Gateway invocation: %s %s",
             func_arn,
             method or "GET",
             path,
         )
-        asynchronous = not config.SYNCHRONOUS_API_GATEWAY_EVENTS
+        asynchronous = headers.get("X-Amz-Invocation-Type") == "'Event'"
         inv_result = run_lambda(
             func_arn=func_arn,
             event=event,
@@ -663,56 +443,6 @@ def process_sns_notification(
     return inv_result.result
 
 
-def process_kinesis_records(records, stream_name):
-    def chunks(lst, n):
-        # Yield successive n-sized chunks from lst.
-        for i in range(0, len(lst), n):
-            yield lst[i : i + n]
-
-    # feed records into listening lambdas
-    try:
-        stream_arn = aws_stack.kinesis_stream_arn(stream_name)
-        sources = get_event_sources(source_arn=stream_arn)
-        for source in sources:
-            arn = source["FunctionArn"]
-            for chunk in chunks(records, source["BatchSize"]):
-                shard_id = "shardId-000000000000"
-                event = {
-                    "Records": [
-                        {
-                            "eventID": "{0}:{1}".format(shard_id, rec["sequenceNumber"]),
-                            "eventSourceARN": stream_arn,
-                            "eventSource": "aws:kinesis",
-                            "eventVersion": "1.0",
-                            "eventName": "aws:kinesis:record",
-                            "invokeIdentityArn": "arn:aws:iam::{0}:role/lambda-role".format(
-                                TEST_AWS_ACCOUNT_ID
-                            ),
-                            "awsRegion": aws_stack.get_region(),
-                            "kinesis": rec,
-                        }
-                        for rec in chunk
-                    ]
-                }
-                lock_discriminator = None
-                if not config.SYNCHRONOUS_KINESIS_EVENTS:
-                    lock_discriminator = f"{stream_arn}/{shard_id}"
-                    lambda_executors.LAMBDA_ASYNC_LOCKS.assure_lock_present(
-                        lock_discriminator, BoundedSemaphore(source["ParallelizationFactor"])
-                    )
-                run_lambda(
-                    func_arn=arn,
-                    event=event,
-                    context={},
-                    asynchronous=not config.SYNCHRONOUS_KINESIS_EVENTS,
-                    lock_discriminator=lock_discriminator,
-                )
-    except Exception as e:
-        LOG.warning(
-            "Unable to run Lambda function on Kinesis records: %s %s", e, traceback.format_exc()
-        )
-
-
 def get_event_sources(func_name=None, source_arn=None):
     result = []
     for region, details in LambdaRegion.regions().items():
@@ -2405,13 +2135,13 @@ def validate_lambda_config():
         config.LAMBDA_DOCKER_NETWORK
         and config.is_in_docker
         and config.LAMBDA_DOCKER_NETWORK
-        not in DOCKER_CLIENT.get_networks(bootstrap.get_main_container_name())
+        not in DOCKER_CLIENT.get_networks(get_main_container_name())
     ):
         LOG.warning(
             "Your specified LAMBDA_DOCKER_NETWORK '%s' is not connected to the main LocalStack container '%s'. "
             "Lambda functionality might be severely limited.",
             config.LAMBDA_DOCKER_NETWORK,
-            bootstrap.get_main_container_name(),
+            get_main_container_name(),
         )
 
 
