@@ -6,16 +6,19 @@ import time
 from collections import defaultdict
 from functools import lru_cache
 from io import BytesIO
-from typing import Any, Dict, List, Union
+from typing import Any, Dict, List, Optional, Union
 
 from flask import Response
 
 from localstack import config
-from localstack.utils import bootstrap
 from localstack.utils.aws import aws_stack
 from localstack.utils.aws.aws_models import LambdaFunction
 from localstack.utils.aws.aws_responses import flask_error_response_json
 from localstack.utils.common import short_uid, to_str
+from localstack.utils.container_networking import (
+    get_endpoint_for_network,
+    get_main_container_network,
+)
 from localstack.utils.docker_utils import DOCKER_CLIENT
 
 LOG = logging.getLogger(__name__)
@@ -38,6 +41,7 @@ LAMBDA_RUNTIME_JAVA11 = "java11"
 LAMBDA_RUNTIME_DOTNETCORE2 = "dotnetcore2.0"
 LAMBDA_RUNTIME_DOTNETCORE21 = "dotnetcore2.1"
 LAMBDA_RUNTIME_DOTNETCORE31 = "dotnetcore3.1"
+LAMBDA_RUNTIME_DOTNET6 = "dotnet6"
 LAMBDA_RUNTIME_GOLANG = "go1.x"
 LAMBDA_RUNTIME_RUBY = "ruby"
 LAMBDA_RUNTIME_RUBY25 = "ruby2.5"
@@ -55,6 +59,7 @@ DOTNET_LAMBDA_RUNTIMES = [
     LAMBDA_RUNTIME_DOTNETCORE2,
     LAMBDA_RUNTIME_DOTNETCORE21,
     LAMBDA_RUNTIME_DOTNETCORE31,
+    LAMBDA_RUNTIME_DOTNET6,
 ]
 
 # IP address of main Docker container (lazily initialized)
@@ -126,24 +131,35 @@ def is_provided_runtime(runtime_details: Union[LambdaFunction, str]) -> bool:
     return runtime.startswith("provided")
 
 
+def format_name_to_path(handler_name: str, delimiter: str, extension: str):
+    file_path = handler_name.rpartition(delimiter)[0]
+    if delimiter == ":":
+        file_path = file_path.split(delimiter)[0]
+
+    if os.path.sep not in file_path:
+        file_path = file_path.replace(".", os.path.sep)
+
+    if file_path.startswith(f".{os.path.sep}"):
+        file_path = file_path[2:]
+
+    return f"{file_path}{extension}"
+
+
 def get_handler_file_from_name(handler_name: str, runtime: str = None):
     runtime = runtime or LAMBDA_DEFAULT_RUNTIME
+
     if runtime.startswith(LAMBDA_RUNTIME_PROVIDED):
         return "bootstrap"
-    delimiter = "."
     if runtime.startswith(LAMBDA_RUNTIME_NODEJS):
-        file_ext = ".js"
-    elif runtime.startswith(LAMBDA_RUNTIME_GOLANG):
-        file_ext = ""
-    elif runtime.startswith(tuple(DOTNET_LAMBDA_RUNTIMES)):
-        file_ext = ".dll"
-        delimiter = ":"
-    elif runtime.startswith(LAMBDA_RUNTIME_RUBY):
-        file_ext = ".rb"
-    else:
-        handler_name = handler_name.rpartition(delimiter)[0].replace(delimiter, os.path.sep)
-        file_ext = ".py"
-    return "%s%s" % (handler_name.split(delimiter)[0], file_ext)
+        return format_name_to_path(handler_name, ".", ".js")
+    if runtime.startswith(LAMBDA_RUNTIME_GOLANG):
+        return handler_name
+    if runtime.startswith(tuple(DOTNET_LAMBDA_RUNTIMES)):
+        return format_name_to_path(handler_name, ":", ".dll")
+    if runtime.startswith(LAMBDA_RUNTIME_RUBY):
+        return format_name_to_path(handler_name, ".", ".rb")
+
+    return format_name_to_path(handler_name, ".", ".py")
 
 
 def is_java_lambda(lambda_details):
@@ -176,53 +192,18 @@ def store_lambda_logs(
     return store_cloudwatch_logs(log_group_name, log_stream_name, log_output, invocation_time)
 
 
-def get_main_endpoint_from_container():
+def get_main_endpoint_from_container() -> str:
     global DOCKER_MAIN_CONTAINER_IP
-    if not config.HOSTNAME_FROM_LAMBDA and DOCKER_MAIN_CONTAINER_IP is None:
-        DOCKER_MAIN_CONTAINER_IP = False
-        container_name = bootstrap.get_main_container_name()
-        try:
-            if config.is_in_docker:
-                DOCKER_MAIN_CONTAINER_IP = DOCKER_CLIENT.get_container_ipv4_for_network(
-                    container_name_or_id=container_name,
-                    container_network=get_container_network_for_lambda(),
-                )
-            else:
-                # default gateway for the network should be the host
-                # (only under Linux - otherwise fall back to DOCKER_HOST_FROM_CONTAINER below)
-                if config.is_in_linux:
-                    DOCKER_MAIN_CONTAINER_IP = DOCKER_CLIENT.inspect_network(
-                        get_container_network_for_lambda()
-                    )["IPAM"]["Config"][0]["Gateway"]
-            LOG.info("Determined main container target IP: %s", DOCKER_MAIN_CONTAINER_IP)
-        except Exception as e:
-            LOG.info(
-                'Unable to get IP address of main Docker container "%s": %s', container_name, e
-            )
-    # return (1) predefined endpoint host, or (2) main container IP, or (3) Docker host (e.g., bridge IP)
-    return (
-        config.HOSTNAME_FROM_LAMBDA or DOCKER_MAIN_CONTAINER_IP or config.DOCKER_HOST_FROM_CONTAINER
-    )
+    if config.HOSTNAME_FROM_LAMBDA:
+        return config.HOSTNAME_FROM_LAMBDA
+    return get_endpoint_for_network(network=get_container_network_for_lambda())
 
 
-def get_container_network_for_lambda():
+def get_container_network_for_lambda() -> str:
     global LAMBDA_CONTAINER_NETWORK
     if config.LAMBDA_DOCKER_NETWORK:
         return config.LAMBDA_DOCKER_NETWORK
-    if LAMBDA_CONTAINER_NETWORK is None:
-        try:
-            if config.is_in_docker:
-                networks = DOCKER_CLIENT.get_networks(bootstrap.get_main_container_name())
-                LAMBDA_CONTAINER_NETWORK = networks[0]
-            else:
-                LAMBDA_CONTAINER_NETWORK = (
-                    "bridge"  # use the default bridge network in case of host mode
-                )
-            LOG.info("Determined lambda container network: %s", LAMBDA_CONTAINER_NETWORK)
-        except Exception as e:
-            container_name = bootstrap.get_main_container_name()
-            LOG.info('Unable to get network name of main container "%s": %s', container_name, e)
-    return LAMBDA_CONTAINER_NETWORK
+    return get_main_container_network()
 
 
 def rm_docker_container(container_name_or_id, check_existence=False, safe=False):
@@ -262,7 +243,9 @@ def get_zip_bytes(function_code):
             s3_client.download_fileobj(function_code["S3Bucket"], function_code["S3Key"], bytes_io)
             zip_file_content = bytes_io.getvalue()
         except Exception as e:
-            raise ClientError("Unable to fetch Lambda archive from S3: %s" % e, 404)
+            s3_key = str(function_code.get("S3Key") or "")
+            s3_url = f's3://{function_code["S3Bucket"]}{s3_key if s3_key.startswith("/") else f"/{s3_key}"}'
+            raise ClientError(f"Unable to fetch Lambda archive from {s3_url}: {e}", 404)
     elif "ZipFile" in function_code:
         zip_file_content = function_code["ZipFile"]
         zip_file_content = base64.b64decode(zip_file_content)
@@ -296,3 +279,12 @@ def error_response(msg, code=500, error_type="InternalFailure"):
     if code != 404:
         LOG.debug(msg)
     return flask_error_response_json(msg, code=code, error_type=error_type)
+
+
+def generate_lambda_arn(
+    account_id: int, region: str, fn_name: str, qualifier: Optional[str] = None
+):
+    if qualifier:
+        return f"arn:aws:lambda:{region}:{account_id}:function:{fn_name}:{qualifier}"
+    else:
+        return f"arn:aws:lambda:{region}:{account_id}:function:{fn_name}"
