@@ -9,16 +9,14 @@ import requests
 from werkzeug.exceptions import NotFound
 
 from localstack import config, constants
-from localstack.http import Router
+from localstack.http import Request, Response, Router
 from localstack.http.adapters import RouterListener
 from localstack.http.dispatcher import resource_dispatcher
-from localstack.services.infra import terminate_all_processes_in_docker
+from localstack.services.infra import SHUTDOWN_INFRA, terminate_all_processes_in_docker
 from localstack.utils.collections import merge_recursive
 from localstack.utils.files import load_file
 from localstack.utils.functions import call_safe
-from localstack.utils.http import parse_request_data
 from localstack.utils.json import parse_json_or_yaml
-from localstack.utils.strings import to_str
 
 LOG = logging.getLogger(__name__)
 
@@ -35,13 +33,19 @@ class HealthResource:
         self.service_manager = service_manager
         self.state = {}
 
-    def on_post(self, request):
-        data = request.json()
+    def on_post(self, request: Request):
+        data = request.get_json(True, True)
+        if not data:
+            return Response("invalid request", 400)
+
         # backdoor API to support restarting the instance
         if data.get("action") in ["kill", "restart"]:
             terminate_all_processes_in_docker()
+            SHUTDOWN_INFRA.set()
 
-    def on_get(self, request):
+        return Response("ok", 200)
+
+    def on_get(self, request: Request):
         path = request.path
 
         reload = "reload" in path
@@ -59,8 +63,8 @@ class HealthResource:
         result["version"] = constants.VERSION
         return result
 
-    def on_put(self, request):
-        data = json.loads(to_str(request.data or "{}"))
+    def on_put(self, request: Request):
+        data = request.get_json(True, True) or {}
 
         # keys like "features:initScripts" should be interpreted as ['features']['initScripts']
         state = defaultdict(dict)
@@ -107,15 +111,11 @@ class CloudFormationUi:
     def on_get(self, request):
         from localstack.utils.aws.aws_stack import get_valid_regions
 
-        path = request.path
-        data = request.data
-        headers = request.headers
-
         deploy_html_file = os.path.join(
             constants.MODULE_MAIN_PATH, "services", "cloudformation", "deploy.html"
         )
         deploy_html = load_file(deploy_html_file)
-        req_params = parse_request_data("GET", path, data, headers)
+        req_params = request.values
         params = {
             "stackName": "stack1",
             "templateBody": "{}",
@@ -127,7 +127,7 @@ class CloudFormationUi:
         if download_url:
             try:
                 LOG.debug("Attempting to download CloudFormation template URL: %s", download_url)
-                template_body = to_str(requests.get(download_url).content)
+                template_body = requests.get(download_url).text
                 template_body = parse_json_or_yaml(template_body)
                 params["templateBody"] = json.dumps(template_body)
             except Exception as e:
@@ -138,7 +138,8 @@ class CloudFormationUi:
         # using simple string replacement here, for simplicity (could be replaced with, e.g., jinja)
         for key, value in params.items():
             deploy_html = deploy_html.replace(f"<{key}>", value)
-        return deploy_html
+
+        return Response(deploy_html, mimetype="text/html")
 
 
 class DiagnoseResource:
